package app.main;

import ryulib.OnNotifyEventListener;
import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.game.GamePlatformInfo;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.view.MotionEvent;

public class Block extends GameControl {

	public Block(GameControlGroup gameControlGroup) {
		super(gameControlGroup);
		
		gameControlGroup.addControl(this);
	}
	
	private int _Size = 0;
	private int _No = 0;
	private Point _Position = new Point(0, 0);
	private boolean _IsBlankBlock = false;
	private OnNotifyEventListener _OnClick = null;

	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		Paint paint = platformInfo.getPaint();
		Canvas canvas = platformInfo.getCanvas();
		
		if (isBlankBlock()) {
			paint.setARGB(255, 0, 0, 0);
			canvas.drawRect(getBoundary().getRect(), paint);
			
			return;
		}

		paint.setARGB(255, 255, 255, 255);
		canvas.drawRect(getBoundary().getRect(), paint);

		paint.setARGB(255, 0, 0, 0);
		getBoundary().drawTextCenter(canvas, paint, Integer.toString(_No));
	}
	
	@Override
    protected boolean onTouchEvent(GamePlatformInfo platformInfo, MotionEvent event) {
		if (isBlankBlock()) return false;

		int x = (int) event.getX();
		int y = (int) event.getY();
		
		if (event.getAction() == MotionEvent.ACTION_DOWN) {
			if (getBoundary().isMyArea(x, y)) {
				if (_OnClick != null) _OnClick.onNotify(this);
				return true;
			}
		}

		return false;
    }
	
	private void changeBoundary() {
		getBoundary().setBoundary(
				_Position.x*_Size + 1, 
				_Position.y*_Size + 1, 
				(_Position.x+1)*_Size - 1, 
				(_Position.y+1)*_Size - 1 
		);
	}

	public void setSize(int _Size) {
		this._Size = _Size;
		changeBoundary();
	}
	
	public int getSize() {
		return _Size;
	}

	public void setNo(int _No) {
		this._No = _No;
	}

	public int getNo() {
		return _No;
	}

	public void setPosition(Point value) {
		_Position.set(value.x, value.y);
		changeBoundary();
	}

	public void setPosition(int x, int y) {
		this._Position.x = x;
		this._Position.y = y;
		changeBoundary();
	}

	public Point getPosition() {
		return _Position;
	}

	public void setIsBlankBlock(boolean _IsBlankBlock) {
		this._IsBlankBlock = _IsBlankBlock;
	}

	public boolean isBlankBlock() {
		return _IsBlankBlock;
	}

	public void setOnClick(OnNotifyEventListener _OnClick) {
		this._OnClick = _OnClick;
	}

	public OnNotifyEventListener getOnClick() {
		return _OnClick;
	}
	
}
