package app.main;

import java.util.Random;

import ryulib.Direction;
import ryulib.OnNotifyEventListener;
import android.graphics.Point;

public class Board {

	public static Board Instance = null;	
	
	public static void createInstance() {
		Instance = new Board();
		Instance.init();
	}
	
	private Board() { }
	
	private int _BlockSize = 64;
	private int _Width = 4;
	private int _Height = 4;	
	private Block[][] _Blocks = null;
	private Block _Blank = null;
	
	private OnNotifyEventListener _OnBlockClick = new OnNotifyEventListener() {
		@Override
		public void onNotify(Object sender) {
			slide(((Block) sender).getPosition()); 
		}
	};
	
	private void init() {
		_Blocks = new Block[_Width][_Height];

		for (int y=0; y<_Height; y++) {
			for (int x=0; x<_Width; x++) {
				_Blocks[x][y] = new Block(Platform.Instance.getGameControlGroup());
				_Blocks[x][y].setSize(_BlockSize);  
				_Blocks[x][y].setNo(x + _Width*y + 1);  
				_Blocks[x][y].setPosition(x, y);  
				_Blocks[x][y].setIsBlankBlock(false);
				_Blocks[x][y].setOnClick(_OnBlockClick);
			}	
		}
		
		_Blank = _Blocks[_Width-1][_Height-1];
		_Blank.setIsBlankBlock(true);
	}

	private Random _Random = new Random();
	
	public void shuffle() {
		Point point = new Point();
		
		for (int i=0; i<1000; i++) {
			point.set(_Random.nextInt(_Width), _Random.nextInt(_Height));
			if (_Blocks[point.x][point.y].isBlankBlock() == false) 
				slide(point);
		}
	}
	
	public void slide(Point point) {
		switch (getBlankDirection(point)) {
		case Left: moveBlockLeft(point); break;
		case Right: moveBlockRight(point); break;
		case Up: moveBlockUp(point); break;
		case Down: moveBlockDown(point); break;
		}
	}
	
	public Direction getBlankDirection(Point point) {
		Direction result = Direction.NoWhere;
		
		if (point.x == _Blank.getPosition().x) {
			if (point.y > _Blank.getPosition().y) result = Direction.Up;
			else result = Direction.Down;
		} else if (point.y == _Blank.getPosition().y) {
			if (point.x > _Blank.getPosition().x) result = Direction.Left;
			else result = Direction.Right;
		}
		
		return result;
	}
	
	private void moveBlockLeft(Point point) {
		int blankX = _Blank.getPosition().x;
		int blankY = _Blank.getPosition().y;

		int count = point.x - blankX;
		
		for (int i=1; i<=count; i++) {
			int x = blankX + i;
			if (x > 0) swapBlocks(_Blocks[x][blankY], _Blocks[x-1][blankY]);
		}
	}
	
	private void swapBlocks(Block a, Block b) {
		_Blocks[a.getPosition().x][a.getPosition().y] = b;
		_Blocks[b.getPosition().x][b.getPosition().y] = a;
		
		int tempX = a.getPosition().x;
		int tempY = a.getPosition().y;
		a.setPosition(b.getPosition());
		b.setPosition(tempX, tempY);
	}

	private void moveBlockRight(Point point) {
		int blankX = _Blank.getPosition().x;
		int blankY = _Blank.getPosition().y;

		int count = blankX - point.x;
		
		for (int i=1; i<=count; i++) {
			int x = blankX - i;
			if (x < (_Width-1)) swapBlocks(_Blocks[x][blankY], _Blocks[x+1][blankY]);
		}
	}
	
	private void moveBlockUp(Point point) {
		int blankX = _Blank.getPosition().x;
		int blankY = _Blank.getPosition().y;

		int count = point.y - blankY;
		
		for (int i=1; i<=count; i++) {
			int y = blankY + i;
			if (y > 0) swapBlocks(_Blocks[blankX][y], _Blocks[blankX][y-1]);
		}
	}
	
	private void moveBlockDown(Point point) {
		int blankX = _Blank.getPosition().x;
		int blankY = _Blank.getPosition().y;

		int count = blankY - point.y;
		
		for (int i=1; i<=count; i++) {
			int y = blankY - i;
			if (y < (_Height-1)) swapBlocks(_Blocks[blankX][y], _Blocks[blankX][y+1]);
		}
	}
	
}
